@@ -7,12 +7,12 @@ import urllib.parse
 import ldap3
 from ldap3.core.exceptions import LDAPException
 
-from trytond.i18n import gettext
-from trytond.model.exceptions import AccessError
-from trytond.transaction import Transaction
-from trytond.pool import PoolMeta
 from trytond.config import config, parse_uri
 from trytond.exceptions import LoginException
+from trytond.i18n import gettext
+from trytond.model.exceptions import AccessError
+from trytond.pool import PoolMeta
+from trytond.transaction import without_check_access
 
 logger = logging.getLogger(__name__)
 section = 'ldap_authentication'
@@ -47,6 +47,8 @@ def ldap_server():
     else:
         scheme, port = 'ldap', 389
         tls = None
+        if 'tls' in uri.scheme:
+            tls = ldap3.Tls(validate=ssl.CERT_REQUIRED)
     return ldap3.Server('%s://%s:%s' % (
             scheme, uri.hostname, uri.port or port), tls=tls)
 
@@ -66,7 +68,9 @@ class User(metaclass=PoolMeta):
         scope = {
             'base': ldap3.BASE,
             'onelevel': ldap3.LEVEL,
+            'one': ldap3.LEVEL,
             'subtree': ldap3.SUBTREE,
+            'sub': ldap3.SUBTREE,
             }[scope]
         uid = config.get(section, 'uid', default='uid')
         if filter_:
@@ -82,7 +86,12 @@ class User(metaclass=PoolMeta):
             # XXX find better way to get the password
             bindpass = config.get(section, 'bind_pass')
 
-        with ldap3.Connection(server, bindname, bindpass) as con:
+        bind_method = ldap3.AUTO_BIND_DEFAULT
+        if server.ssl is False and server.tls is not None:
+            bind_method = ldap3.AUTO_BIND_TLS_BEFORE_BIND
+
+        with ldap3.Connection(
+                server, bindname, bindpass, auto_bind=bind_method) as con:
             con.search(dn, filter_, search_scope=scope, attributes=attrs)
             result = con.entries
             if result and len(result) > 1:
@@ -91,6 +100,7 @@ class User(metaclass=PoolMeta):
                 for e in result]
 
     @classmethod
+    @without_check_access
     def _check_passwd_ldap_user(cls, logins):
         find = False
         try:
@@ -115,8 +125,7 @@ class User(metaclass=PoolMeta):
             if values.get('password') and 'login' in values:
                 tocheck.append(values['login'])
         if tocheck:
-            with Transaction().set_context(_check_access=False):
-                cls._check_passwd_ldap_user(tocheck)
+            cls._check_passwd_ldap_user(tocheck)
         return super(User, cls).create(vlist)
 
     @classmethod
@@ -141,7 +150,11 @@ class User(metaclass=PoolMeta):
                 users = cls.ldap_search_user(login, server, attrs=[uid])
                 if users and len(users) == 1:
                     [(dn, attrs)] = users
-                    with ldap3.Connection(server, dn, password) as con:
+                    with ldap3.Connection(
+                            server, dn, password,
+                            auto_bind=ldap3.AUTO_BIND_NONE) as con:
+                        if server.ssl is False and server.tls is not None:
+                            con.start_tls()
                         if (password and con.bind()):
                             # Use ldap uid so we always get the right case
                             login = attrs.get(uid, [login])[0]
